/**
 * Copyright (c) 2013, 2014, 2016 Jelle Hellings.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY JELLE HELLINGS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
 * EVENT SHALL THE FOUNDATION OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef INCLUDE_GRAPH_HPP_
#define INCLUDE_GRAPH_HPP_
#include <cstdint>
#include <vector>
#include <utility>

typedef std::uint32_t node_id_type;
typedef std::uint32_t relation_id_type;

/* */
struct edge_type {
    edge_type(const node_id_type from, const node_id_type to) : from(from),  to(to) { }
    node_id_type from;
    node_id_type to;

    bool operator<(const edge_type& other) const {
        return (from < other.from) ||
               ((from == other.from) && (to < other.to));
    }

    bool operator==(const edge_type& other) const {
        return (from == other.from) && (to == other.to);
    }
};

typedef std::vector<edge_type> raw_relation;

/* Simple class representation. */
struct graph {
    /* Name of the graph. */
    std::string name;

    /* Node names (index in vector is the node id). */
    std::vector<std::string> node_names;

    /* Edge-label names (index in vector is the relation id). */
    std::vector<std::string> relation_names;

    /* A list of relations: relation at index i is the relation for the
     * edge-label with relation id i. Each relation is a list of edge-pairs,
     * in which an edge pair is a pair of node ids. */
    std::vector<raw_relation> relations;

    /* Return the number of nodes. */
    node_id_type num_nodes() const {
        return static_cast<node_id_type>(node_names.size());
    }

    /* Return the number of relations. */
    relation_id_type num_relations() const {
        return static_cast<relation_id_type>(relation_names.size());
    }
};

#endif