/**
 * Copyright (c) 2016 Jelle Hellings.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY JELLE HELLINGS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
 * EVENT SHALL THE FOUNDATION OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef INCLUDE_OVERFLOW_HPP_
#define INCLUDE_OVERFLOW_HPP_
#include <limits>
#include <utility>

/**
 * The function overflow_multiply returns a pair (value, bool) such that value
 * is the product of the input and the boolean is true if and only if computing
 * the product caused an overflow (the result is wrong).
 */
template<class Type, class SingleNumber>
std::pair<Type, bool> overflow_multiply(const SingleNumber number) {
    return std::pair<Type, bool>(static_cast<Type>(number),
                                 number > std::numeric_limits<Type>::max());
}

template<class Type, class FirstNumber, class SecondNumber, class... Numbers>
std::pair<Type, bool> overflow_multiply(const FirstNumber lhs, const SecondNumber rhs, const Numbers... rest) {
    auto value_lhs = overflow_multiply<Type>(lhs);
    auto value_rhs = overflow_multiply<Type>(rhs, rest...);
    bool overflow = (std::numeric_limits<Type>::max() / value_lhs.first) < value_rhs.first;
    return std::pair<Type, bool>(value_lhs.first * value_rhs.first,
                                 value_lhs.second || value_rhs.second || overflow);
}

#endif