/**
 * Copyright (c) 2014, 2016 Jelle Hellings.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY JELLE HELLINGS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
 * EVENT SHALL THE FOUNDATION OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <cstdint>
#include <iostream>
#include <stdexcept>
#include "string_parser.hpp"

/* Return true if ch is a newline character. */
bool isnewline(char ch) {
    return ch == '\n' || ch == '\r';
}


string_reader::position_info string_reader::query_position(size_type position) const {
    size_type line = 1;
    size_type column = 1;
    size_type pos_line = 0;

    size_type i = 0;
    for (size_type last = std::min(position, length()); i < last; ++i) {
        if (isnewline(source[i])) {
            if (i == 0 || !isnewline(source[i - 1]) || source[i] == source[i - 1]) {
                ++line;
            }
            pos_line = i + 1;
            column = 0;
        }
        ++column;
    }

    return position_info(line, column, pos_line);
}

void string_reader::throw_error(const std::string& message) const {
    auto info = query_position(position());
    auto last_line = slice(info.start_line, std::min(position(), length()));
    std::cerr << "parse message: " << message
                                   << " at (" << info.line << ", " << info.column << ") \n"
                                         "\t" << last_line << "\n";
    throw std::runtime_error("parse error");
}